@@ -6,8 +6,7 @@ PyObject *node_new_internal(ModuleState *state, TSNode node, PyObject *tree) {
         return NULL;
     }
     self->node = node;
-    Py_INCREF(tree);
-    self->tree = tree;
+    self->tree = Py_NewRef(tree);
     self->children = NULL;
     return PyObject_Init((PyObject *)self, state->node_type);
 }
@@ -53,8 +52,7 @@ PyObject *node_walk(Node *self, PyObject *Py_UNUSED(args)) {
         return NULL;
     }
 
-    Py_INCREF(self->tree);
-    tree_cursor->tree = self->tree;
+    tree_cursor->tree = Py_NewRef(self->tree);
     tree_cursor->node = NULL;
     tree_cursor->cursor = ts_tree_cursor_new(self->node);
     return PyObject_Init((PyObject *)tree_cursor, state->tree_cursor_type);
@@ -223,6 +221,27 @@ PyObject *node_field_name_for_child(Node *self, PyObject *args) {
     return PyUnicode_FromString(field_name);
 }
 
+PyObject *node_field_name_for_named_child(Node *self, PyObject *args) {
+    long index;
+    if (!PyArg_ParseTuple(args, "l:field_name_for_named_child", &index)) {
+        return NULL;
+    }
+    if (index < 0) {
+        PyErr_SetString(PyExc_ValueError, "child index must be positive");
+        return NULL;
+    }
+    if ((uint32_t)index >= ts_node_child_count(self->node)) {
+        PyErr_SetString(PyExc_IndexError, "child index out of range");
+        return NULL;
+    }
+
+    const char *field_name = ts_node_field_name_for_named_child(self->node, index);
+    if (field_name == NULL) {
+        Py_RETURN_NONE;
+    }
+    return PyUnicode_FromString(field_name);
+}
+
 PyObject *node_descendant_for_byte_range(Node *self, PyObject *args) {
     ModuleState *state = GET_MODULE_STATE(self);
     uint32_t start_byte, end_byte;
@@ -285,6 +304,9 @@ PyObject *node_child_containing_descendant(Node *self, PyObject *args) {
     if (!PyArg_ParseTuple(args, "O!:child_containing_descendant", &descendant, state->node_type)) {
         return NULL;
     }
+    if (REPLACE("child_containing_descendant", "child_with_descendant") < 0) {
+        return NULL;
+    }
 
     TSNode child = ts_node_child_containing_descendant(self->node, descendant);
     if (ts_node_is_null(child)) {
@@ -293,6 +315,20 @@ PyObject *node_child_containing_descendant(Node *self, PyObject *args) {
     return node_new_internal(state, child, self->tree);
 }
 
+PyObject *node_child_with_descendant(Node *self, PyObject *args) {
+    ModuleState *state = GET_MODULE_STATE(self);
+    TSNode descendant;
+    if (!PyArg_ParseTuple(args, "O!:child_with_descendant", &descendant, state->node_type)) {
+        return NULL;
+    }
+
+    TSNode child = ts_node_child_with_descendant(self->node, descendant);
+    if (ts_node_is_null(child)) {
+        Py_RETURN_NONE;
+    }
+    return node_new_internal(state, child, self->tree);
+}
+
 PyObject *node_get_id(Node *self, void *Py_UNUSED(payload)) {
     return PyLong_FromVoidPtr((void *)self->node.id);
 }
@@ -399,8 +435,7 @@ PyObject *node_get_end_point(Node *self, void *Py_UNUSED(payload)) {
 PyObject *node_get_children(Node *self, void *Py_UNUSED(payload)) {
     ModuleState *state = GET_MODULE_STATE(self);
     if (self->children) {
-        Py_INCREF(self->children);
-        return self->children;
+        return Py_NewRef(self->children);
     }
 
     uint32_t length = ts_node_child_count(self->node);
@@ -421,9 +456,8 @@ PyObject *node_get_children(Node *self, void *Py_UNUSED(payload)) {
             }
         } while (ts_tree_cursor_goto_next_sibling(&state->default_cursor));
     }
-    Py_INCREF(result);
-    self->children = result;
-    return result;
+    self->children = Py_NewRef(result);
+    return self->children;
 }
 
 PyObject *node_get_named_children(Node *self, void *payload) {
@@ -444,8 +478,7 @@ PyObject *node_get_named_children(Node *self, void *payload) {
     for (uint32_t i = 0, j = 0; i < length; ++i) {
         PyObject *child = PyList_GetItem(self->children, i);
         if (ts_node_is_named(((Node *)child)->node)) {
-            Py_INCREF(child);
-            if (PyList_SetItem(result, j++, child)) {
+            if (PyList_SetItem(result, j++, Py_NewRef(child))) {
                 Py_DECREF(result);
                 return NULL;
             }
@@ -600,6 +633,9 @@ PyDoc_STRVAR(node_children_by_field_name_doc, "children_by_field_name(self, name
 PyDoc_STRVAR(node_field_name_for_child_doc,
              "field_name_for_child(self, child_index, /)\n--\n\n"
              "Get the field name of this node's child at the given index.");
+PyDoc_STRVAR(node_field_name_for_named_child_doc,
+             "field_name_for_child(self, child_index, /)\n--\n\n"
+             "Get the field name of this node's *named* child at the given index.");
 PyDoc_STRVAR(node_descendant_for_byte_range_doc,
              "descendant_for_byte_range(self, start_byte, end_byte, /)\n--\n\n"
              "Get the smallest node within this node that spans the given byte range.");
@@ -614,7 +650,11 @@ PyDoc_STRVAR(node_named_descendant_for_point_range_doc,
              "Get the smallest *named* node within this node that spans the given point range.");
 PyDoc_STRVAR(node_child_containing_descendant_doc,
              "child_containing_descendant(self, descendant, /)\n--\n\n"
-             "Get the child of the node that contains the given descendant.");
+             "Get the child of the node that contains the given descendant." DOC_ATTENTION
+             "This will not return the descendant if it is a direct child of this node.");
+PyDoc_STRVAR(node_child_with_descendant_doc,
+             "child_with_descendant(self, descendant, /)\n--\n\n"
+             "Get the node that contains the given descendant.");
 
 static PyMethodDef node_methods[] = {
     {
@@ -671,6 +711,12 @@ static PyMethodDef node_methods[] = {
         .ml_flags = METH_VARARGS,
         .ml_doc = node_field_name_for_child_doc,
     },
+    {
+        .ml_name = "field_name_for_named_child",
+        .ml_meth = (PyCFunction)node_field_name_for_named_child,
+        .ml_flags = METH_VARARGS,
+        .ml_doc = node_field_name_for_named_child_doc,
+    },
     {
         .ml_name = "descendant_for_byte_range",
         .ml_meth = (PyCFunction)node_descendant_for_byte_range,
@@ -701,6 +747,12 @@ static PyMethodDef node_methods[] = {
         .ml_flags = METH_VARARGS,
         .ml_doc = node_child_containing_descendant_doc,
     },
+    {
+        .ml_name = "child_with_descendant",
+        .ml_meth = (PyCFunction)node_child_with_descendant,
+        .ml_flags = METH_VARARGS,
+        .ml_doc = node_child_with_descendant_doc,
+    },
     {NULL},
 };
 
